#!/usr/bin/python
# -*- coding: latin-1 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

"Módulo para utilizar el servicio web Constatación de Comprobantes de AFIP"

# Información adicional y documentación:
# http://www.sistemasagiles.com.ar/trac/wiki/ConstatacionComprobantes

__author__ = "Mariano Reingart (reingart@gmail.com)"
__copyright__ = "Copyright (C) 2013-2015 Mariano Reingart"
__license__ = "GPL 3.0"
__version__ = "1.02e"

import sys
from configparser import SafeConfigParser
from .utils import (
    inicializar_y_capturar_excepciones,
    BaseWS,
    get_install_dir,
    leer,
    escribir,
    leer_dbf,
    guardar_dbf,
    N,
    A,
    I,
    json
)


# Constantes (si se usa el script de linea de comandos)
WSDL = "https://wswhomo.afip.gov.ar/WSCDC/service.asmx?WSDL"
HOMO = False
CONFIG_FILE = "rece.ini"

# No debería ser necesario modificar nada despues de esta linea

# definición del formato del archivo de intercambio (sólo para linea de comandos):

ENCABEZADO = [
    ('tipo_reg', 1, A, u"0: encabezado"),
    ('cbte_modo', 4, A, u"Modalidad de autorización (CAI, CAE, CAEA)"),
    ('cuit_emisor', 11, A, u"CUIT del emisor del comprobante"),
    ('pto_vta', 4, N, u"Punto de Venta del comprobante"),
    ('cbte_tipo', 3, N, u"Tipo de comprobante"),
    ('cbte_nro', 8, N, u"Número de comprobante"),
    ('cbte_fch', 8, A, u"Fecha en formato AAAAMMDD"),
    ('imp_total', 15, I, u"Importe total Double (13 + 2)"),
    ('cod_autorizacion', 14, A, u"Número de CAI, CAE, CAEA"),
    ('doc_tipo_receptor', 2, A, u"Tipo de documento del receptor"),
    ('doc_nro_receptor', 20, A, u"N° de documento del receptor"),
    # campos devueltos por AFIP (respuesta)
    ('resultado', 1, A, u"Resultado (A: Aprobado, O: Observado, R: rechazado)"),
    ('fch_proceso', 14, A, u"Fecha y hora de procesamiento"),
    ]

OBSERVACION = [
    ('tipo_reg', 1, A, u"O: observaciones devueltas por AFIP"),
    ('code', 5, N, u"Código de Observación / Error / Evento"),
    ('msg', 255, A, u"Mensaje"),
    ]

EVENTO = ERROR = OBSERVACION        # misma estructura, cambia tipo de registro


class WSCDC(BaseWS):
    "Interfaz para el WebService de Constatación de Comprobantes"
    _public_methods_ = [
        'Conectar', 'SetTicketAcceso', 'DebugLog',
        'AnalizarXml', 'ObtenerTagXml', 'LoadTestXML',
        'ConstatarComprobante', 'Dummy',
        'ConsultarModalidadComprobantes',
        'ConsultarTipoComprobantes',
        'ConsultarTipoDocumentos', 'ConsultarTipoOpcionales',
        'SetParametros', 'SetParametro', 'GetParametro',
    ]
    _public_attrs_ = [
        'Token', 'Sign', 'Cuit', 'ExpirationTime', 'Version',
        'XmlRequest', 'XmlResponse', 'Observaciones', 'Errores',
        'InstallDir', 'Traceback', 'Excepcion', 'ErrMsg', 'Obs',
        'AppServerStatus', 'DbServerStatus', 'AuthServerStatus',
        'Resultado', 'FchProceso', 'Observaciones', 'Obs',
        'FechaCbte', 'CbteNro', 'PuntoVenta', 'ImpTotal',
        'EmisionTipo', 'CAE', 'CAEA', 'CAI',
        'DocTipo', 'DocNro',
        'SoapFault', 'LanzarExcepciones',
    ]
    _readonly_attrs_ = _public_attrs_[3:-1]
    _reg_progid_ = "WSCDC"
    _reg_clsid_ = "{D1B97BDD-A78C-4D51-8999-1D9A5034EC10}"

    # Variables globales para BaseWS:
    HOMO = HOMO
    WSDL = WSDL
    Version = "%s %s" % (__version__, HOMO and 'Homologación' or '')

    def inicializar(self):
        BaseWS.inicializar(self)
        self.AppServerStatus = self.DbServerStatus = self.AuthServerStatus = None
        self.Resultado = self.EmisionTipo = ""
        self.CAI = self.CAE = self.CAEA = self.Vencimiento = ''
        self.CbteNro = self.PuntoVenta = self.ImpTotal = None

    def __analizar_errores(self, ret):
        "Comprueba y extrae errores si existen en la respuesta XML"
        if 'Errors' in ret:
            errores = ret['Errors']
            for error in errores:
                self.Errores.append("%s: %s" % (
                    error['Err']['Code'],
                    error['Err']['Msg'],
                    ))
            self.errores = [{'code': err['Err']['Code'], 'msg': err['Err']['Msg'].replace("\n", "").replace("\r", "")} for err in errores]
            self.ErrCode = ' '.join([str(error['Err']['Code']) for error in errores])
            self.ErrMsg = '\n'.join(self.Errores)

    @inicializar_y_capturar_excepciones
    def Dummy(self):
        "Método Dummy para verificación de funcionamiento de infraestructura"
        result = self.client.ComprobanteDummy()['ComprobanteDummyResult']
        self.AppServerStatus = result['AppServer']
        self.DbServerStatus = result['DbServer']
        self.AuthServerStatus = result['AuthServer']
        self.__analizar_errores(result)
        return True

    @inicializar_y_capturar_excepciones
    def ConstatarComprobante(self, cbte_modo, cuit_emisor, pto_vta, cbte_tipo,
                             cbte_nro, cbte_fch, imp_total, cod_autorizacion,
                             doc_tipo_receptor=None, doc_nro_receptor=None,
                             **kwargs):
        "Método de Constatación de Comprobantes"
        response = self.client.ComprobanteConstatar(
                    Auth={
                        'Token': self.Token.decode('utf8'),
                        'Sign': self.Sign.decode('utf8'),
                        'Cuit': self.Cuit
                    },
                    CmpReq={
                        'CbteModo': cbte_modo,
                        'CuitEmisor': cuit_emisor,
                        'PtoVta': pto_vta,
                        'CbteTipo': cbte_tipo,
                        'CbteNro': cbte_nro,
                        'CbteFch': cbte_fch,
                        'ImpTotal': imp_total,
                        'CodAutorizacion': cod_autorizacion,
                        'DocTipoReceptor': doc_tipo_receptor,
                        'DocNroReceptor': doc_nro_receptor,
                        }
                    )
        result = response['ComprobanteConstatarResult']
        self.__analizar_errores(result)
        if 'CmpResp' in result:
            resp = result['CmpResp']
            self.Resultado = result['Resultado']
            self.FchProceso = result.get('FchProceso', "")
            self.observaciones = []
            for obs in result.get('Observaciones', []):
                self.Observaciones.append("%(Code)s: %(Msg)s" % (obs['Obs']))
                self.observaciones.append({'code': obs['Obs']['Code'], 'msg': obs['Obs']['Msg'].replace("\n", "").replace("\r", "")})
            self.Obs = '\n'.join(self.Observaciones)
            self.FechaCbte = resp.get('CbteFch', "")  # .strftime("%Y/%m/%d")
            self.CbteNro = resp.get('CbteNro', 0)  # 1L
            self.PuntoVenta = resp.get('PtoVta', 0)  # 4000
            self.ImpTotal = str(resp['ImpTotal'])
            self.EmisionTipo = resp['CbteModo']
            self.DocTipo = resp.get('DocTipoReceptor', '')
            self.DocNro = resp.get('DocNroReceptor', '')
            cod_aut = str(resp.get('CodAutorizacion', ""))  # 60423794871430L
            if self.EmisionTipo == 'CAE':
                self.CAE = cod_aut
            elif self.EmisionTipo == 'CAEA':
                self.CAEA = cod_aut
            elif self.EmisionTipo == 'CAI':
                self.CAI = cod_aut
        return True

    @inicializar_y_capturar_excepciones
    def ConsultarModalidadComprobantes(self, sep="|"):
        "Recuperador de modalidades de autorización de comprobantes"
        response = self.client.ComprobantesModalidadConsultar(
            Auth={
                'Token': self.Token.decode('utf8'),
                'Sign': self.Sign.decode('utf8'),
                'Cuit': self.Cuit
            },
        )
        result = response['ComprobantesModalidadConsultarResult']
        self.__analizar_errores(result)
        return [(u"\t%(Cod)s\t%(Desc)s\t" % p['FacModTipo']).replace("\t", sep)
                for p in result['ResultGet']]

    @inicializar_y_capturar_excepciones
    def ConsultarTipoComprobantes(self, sep="|"):
        "Recuperador de valores referenciales de códigos de Tipos de comprobante"
        response = self.client.ComprobantesTipoConsultar(
            Auth={
                'Token': self.Token.decode('utf8'),
                'Sign': self.Sign.decode('utf8'),
                'Cuit': self.Cuit
            },
        )
        result = response['ComprobantesTipoConsultarResult']
        self.__analizar_errores(result)
        return [(u"\t%(Id)s\t%(Desc)s\t" % p['CbteTipo']).replace("\t", sep)
                for p in result['ResultGet']]

    @inicializar_y_capturar_excepciones
    def ConsultarTipoDocumentos(self, sep="|"):
        "Recuperador de valores referenciales de códigos de Tipos de Documentos"
        response = self.client.DocumentosTipoConsultar(
            Auth={
                'Token': self.Token.decode('utf8'),
                'Sign': self.Sign.decode('utf8'),
                'Cuit': self.Cuit
            },
        )
        result = response['DocumentosTipoConsultarResult']
        self.__analizar_errores(result)
        return [(u"\t%(Id)s\t%(Desc)s\t" % p['DocTipo']).replace("\t", sep)
                for p in result['ResultGet']]

    @inicializar_y_capturar_excepciones
    def ConsultarTipoOpcionales(self, sep="|"):
        "Recuperador de valores referenciales de códigos de Tipos de datos Opcionales"
        response = self.client.OpcionalesTipoConsultar(
            Auth={
                'Token': self.Token.decode('utf8'),
                'Sign': self.Sign.decode('utf8'),
                'Cuit': self.Cuit
            },
        )
        result = response['OpcionalesTipoConsultarResult']
        res = result['ResultGet'] if 'ResultGet' in result else []
        self.__analizar_errores(result)
        return [(u"\t%(Id)s\t%(Desc)s\t" % p['OpcionalTipo']).replace("\t", sep)
                for p in res]


# busco el directorio de instalación (global para que no cambie si usan otra dll)
INSTALL_DIR = WSCDC.InstallDir = get_install_dir()


def escribir_archivo(dic, nombre_archivo, agrega=True):
    archivo = open(nombre_archivo, agrega and "a" or "w")
    formatos = [('Encabezado', ENCABEZADO, [dic], 0),
                ('Observacion', OBSERVACION, dic.get('observaciones', []), 'O'),
                ('Eventos', ERROR, dic.get('eventos', []), 'V'),
                ('Error', ERROR, dic.get('errores', []), 'E'),
                ]
    if '--json' in sys.argv:
        json.dump(dic, archivo, sort_keys=True, indent=4)
    elif '--dbf' in sys.argv:
        guardar_dbf(formatos, agrega, conf_dbf)
    else:
        for nombre, formato, registros, tipo_reg in formatos:
            for it in registros:
                it['tipo_reg'] = tipo_reg
                archivo.write(escribir(it, formato))
    archivo.close()


def leer_archivo(nombre_archivo):
    archivo = open(nombre_archivo, "r")
    if '--json' in sys.argv:
        dic = json.load(archivo)
    elif '--dbf' in sys.argv:
        dic = {}
        formatos = [('Encabezado', ENCABEZADO, dic),
                    ]
        leer_dbf(formatos, conf_dbf)
    else:
        dic = {}
        for linea in archivo:
            if str(linea[0])=='0':
                d = leer(linea, ENCABEZADO)
                dic.update(d)
            else:
                print("Tipo de registro incorrecto:", linea[0])
    archivo.close()

    if 'cod_autorizacion' not in dic:
        raise RuntimeError("Archivo de entrada invalido, revise campos y lineas en blanco")

    return dic


def main():
    "Funcion principal para utilizar la interfaz por linea de comando"

    if '--formato' in sys.argv:
        print("Formato:")
        for msg, formato in [('Encabezado', ENCABEZADO),
                             ('Observacion', OBSERVACION),
                             ('Evento', EVENTO), ('Error', ERROR),
                             ]:
            comienzo = 1
            print("=== %s ===" % msg)
            print("|| %-20s || %8s || %9s || %-12s || %-20s ||" % (
                "Campo", "Posición", "Longitud", "Tipo", "Descripción"))
            for fmt in formato:
                clave, longitud, tipo, desc = fmt
                print("|| %-20s || %8d || %9d || %-12s || %-20s ||" % (
                    clave, comienzo, longitud, tipo, desc.encode("latin1")))
                comienzo += longitud
        sys.exit(0)

    # leer configuracion
    global CONFIG_FILE
    if len(sys.argv)>1 and sys.argv[1][0] not in "-/":
        CONFIG_FILE = sys.argv.pop(1)
    config = SafeConfigParser()
    config.read(CONFIG_FILE)
    crt = config.get('WSAA', 'CERT')
    key = config.get('WSAA', 'PRIVATEKEY')
    cuit = config.get('WSCDC', 'CUIT')
    url_wsaa = config.get('WSAA', 'URL') if config.has_option('WSAA','URL') else ""
    url_wscdc = config.get('WSCDC', 'URL') if config.has_option('WSCDC','URL') else ""

    # leo configuración de archivos de intercambio
    ENTRADA = config.get('WSCDC','ENTRADA')
    SALIDA = config.get('WSCDC','SALIDA')
    if config.has_section('DBF'):
        conf_dbf = dict(config.items('DBF'))
    else:
        conf_dbf = {}

    # instanciar la interfaz con el webservice
    wscdc = WSCDC()
    ok = wscdc.Conectar("", url_wscdc)

    if "--dummy" in sys.argv:
        print(wscdc.client.help("ComprobanteDummy"))
        wscdc.Dummy()
        print("AppServerStatus", wscdc.AppServerStatus)
        print("DbServerStatus", wscdc.DbServerStatus)
        print("AuthServerStatus", wscdc.AuthServerStatus)
        sys.exit(0)

    # Gestionar credenciales de acceso con AFIP:
    from wsaa import WSAA
    wsaa = WSAA()
    ta = wsaa.Autenticar("wscdc", crt, key, url_wsaa)
    if not ta:
        sys.exit("Imposible autenticar con WSAA: %s" % wsaa.Excepcion)
    wscdc.SetTicketAcceso(ta)
    wscdc.Cuit = cuit

    if "--constatar" in sys.argv:
        if len(sys.argv) < 8:
            if "--prueba" in sys.argv:
                dic = dict(
                    cbte_modo="CAE",
                    cuit_emisor="20267565393",
                    pto_vta=3,
                    cbte_tipo=6,
                    cbte_nro=1,
                    cbte_fch="20131231",
                    imp_total="1.21",
                    cod_autorizacion="63533727749637",
                    doc_tipo_receptor=99,
                    doc_nro_receptor=0,
                    )
                # escribir archivo de intercambio con datos de prueba:
                escribir_archivo(dic, ENTRADA)
            else:
                # leer archivo de intercambio:
                dic = leer_archivo(ENTRADA)
            # constatar el comprobante
            wscdc.ConstatarComprobante(**dic)
            # actualizar el diccionario con los datos de devueltos por AFIP
            dic.update({'resultado': wscdc.Resultado,
                        'fch_proceso': wscdc.FchProceso,
                        })
            dic['observaciones'] = wscdc.observaciones
            dic['errores'] = wscdc.errores
            escribir_archivo(dic, SALIDA)
        else:
            # usar los datos pasados por linea de comandos:
            wscdc.ConstatarComprobante(*sys.argv[sys.argv.index("--constatar")+1:])

        print("Resultado:", wscdc.Resultado)
        print("Mensaje de Error:", wscdc.ErrMsg)
        print("Observaciones:", wscdc.Obs)

    if "--params" in sys.argv:

        print("=== Modalidad Comprobantes ===")
        print(u'\n'.join(wscdc.ConsultarModalidadComprobantes("||")))
        print("=== Tipo Comprobantes ===")
        print(u'\n'.join(wscdc.ConsultarTipoComprobantes("||")))
        print("=== Tipo Documentos ===")
        print(u'\n'.join(wscdc.ConsultarTipoDocumentos("||")))
        print("=== Tipo Opcionales ===")
        print(u'\n'.join(wscdc.ConsultarTipoOpcionales("||")))
        print("Mensaje de Error:", wscdc.ErrMsg)

if __name__=="__main__":

    if '--register' in sys.argv or '--unregister' in sys.argv:
        import pythoncom
        import win32com.server.register
        win32com.server.register.UseCommandLine(WSCDC)
    elif "/Automate" in sys.argv:
        # MS seems to like /automate to run the class factories.
        import win32com.server.localserver
        #win32com.server.localserver.main()
        # start the server.
        win32com.server.localserver.serve([WSCDC._reg_clsid_])
    else:
        main()




#!/usr/bin/python
# -*- coding: latin-1 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

"M�dulo de Intefase para archivos de texto (MATRIX mercado interno con \
    detalle)"

__author__ = "Mariano Reingart (reingart@gmail.com)"
__copyright__ = "Copyright (C) 2011 Mariano Reingart"
__license__ = "GPL 3.0"
__version__ = "1.31b"

import datetime
import os
import sys
import time
import traceback
from ConfigParser import SafeConfigParser

# revisar la instalaci�n de pyafip.ws:
import wsmtx
from php import SimpleXMLElement, SoapClient, SoapFault, date
from utils import leer, escribir, leer_dbf, guardar_dbf, N, A, I


HOMO = wsmtx.HOMO
DEBUG = False
PDB = False
XML = False
CONFIG_FILE = "rece.ini"

LICENCIA = """
recem.py: Interfaz de texto para generar Facturas Electr�nica MATRIX
Copyright (C) 2010 Mariano Reingart reingart@gmail.com

Este progarma es software libre, se entrega ABSOLUTAMENTE SIN GARANTIA
y es bienvenido a redistribuirlo bajo la licencia GPLv3.

Para informaci�n adicional sobre garant�a, soporte t�cnico comercial
e incorporaci�n/distribuci�n en programas propietarios ver PyAfipWs:
http://www.sistemasagiles.com.ar/trac/wiki/PyAfipWs
"""

# definici�n del formato del archivo de intercambio:

if '--pyfepdf' not in sys.argv:
    TIPOS_REG = '0', '1', '2', '3', '4', '5'
    ENCABEZADO = [
        ('tipo_reg', 1, N),  # 0: encabezado
        ('fecha_cbte', 10, A),
        ('tipo_cbte', 2, N),
        ('punto_vta', 4, N),
        ('cbt_desde', 8, N),
        ('cbt_hasta', 8, N),
        ('concepto', 1, N),  # 1:bienes, 2:servicios,...
        ('tipo_doc', 2, N),  # 80
        ('nro_doc', 11, N),  # 50000000016
        ('imp_total', 15, I, 2),
        ('imp_tot_conc', 15, I, 2),
        ('imp_neto', 15, I, 2),
        ('imp_subtotal', 15, I, 2),
        ('imp_trib', 15, I, 2),
        ('imp_op_ex', 15, I, 2),
        ('moneda_id', 3, A),
        ('moneda_ctz', 10, I, 6),  # 10,6
        ('fecha_venc_pago', 10, A),   # opcional solo conceptos 2 y 3
        ('fecha_serv_desde', 10, A),  # opcional solo conceptos 2 y 3
        ('fecha_serv_hasta', 10, A),  # opcional solo conceptos 2 y 3
        ('cae', 14, A),
        ('fch_venc_cae', 10, A),
        ('resultado', 1, A),
        ('motivos_obs', 1000, A),
        ('err_code', 6, A),
        ('err_msg', 1000, A),
        ('reproceso', 1, A),
        ('emision_tipo', 4, A),
        ('observaciones', 1000, A),  # observaciones (opcional)
        ]

    DETALLE = [
        ('tipo_reg', 1, N),     # 4: detalle item
        ('u_mtx', 10, N),
        ('cod_mtx', 30, A),
        ('codigo', 30, A),
        ('qty', 15, I, 3),      # deber�a ser 18,6 pero el DBF no lo soporta
        ('umed', 3, N),
        ('precio', 15, I, 3),   # deber�a ser 18,6 pero el DBF no lo soporta
        ('bonif', 15, I, 3),
        ('iva_id', 3, N),
        ('imp_iva', 15, I, 2),
        ('imp_subtotal', 15, I, 2),
        ('ds', 4000, A),
        ]

    TRIBUTO = [
        ('tipo_reg', 1, N),     # 1: tributo
        ('tributo_id', 3, A),   # c�digo de otro tributo
        ('desc', 100, A),       # descripci�n
        ('base_imp', 15, I, 2),
        ('alic', 15, I, 2),     # no se usa...
        ('importe', 15, I, 2),
        ]

    IVA = [
        ('tipo_reg', 1, N),     # 2: IVA
        ('iva_id', 3, A),       # c�digo de al�cuota
        ('base_imp', 15, I, 2),  # no se usa...
        ('importe', 15, I, 2),
        ]

    CMP_ASOC = [
        ('tipo_reg', 1, N),     # 3: comprobante asociado
        ('tipo', 3, N),
        ('pto_vta', 4, N),
        ('nro', 8, N),
        ]

else:
    print("!" * 78)
    print("importando formato segun pyfepdf")
    from formatos.formato_txt import (
        ENCABEZADO, DETALLE, PERMISO, CMP_ASOC, IVA, TRIBUTO
    )
    TIPOS_REG = '0', '5', '4', '3', '1'


def autorizar(ws, entrada, salida, informar_caea=False):
    tributos = []
    ivas = []
    cbtasocs = []
    encabezado = []
    detalles = []
    if '/dbf' in sys.argv:
        formatos = [
            ('Encabezado', ENCABEZADO, encabezado),
            ('Tributo', TRIBUTO, tributos),
            ('Iva', IVA, ivas),
            ('Comprobante Asociado', CMP_ASOC, cbtasocs),
            ('Detalles', DETALLE, detalles)]
        dic = leer_dbf(formatos, conf_dbf)
        encabezado = encabezado[0]
    else:
        for linea in entrada:
            if str(linea[0]) == TIPOS_REG[0]:
                encabezado = leer(linea, ENCABEZADO, expandir_fechas=True)
                if 'cbte_nro' in encabezado:
                    print("*" * 80)
                    print("cbte_nro", encabezado['cbte_nro'])
                    encabezado['cbt_desde'] = encabezado['cbte_nro']
                    encabezado['cbt_hasta'] = encabezado['cbte_nro']
                    del encabezado['cbte_nro']
            elif str(linea[0]) == TIPOS_REG[1]:
                tributo = leer(linea, TRIBUTO)
                tributos.append(tributo)
            elif str(linea[0]) == TIPOS_REG[2]:
                iva = leer(linea, IVA)
                ivas.append(iva)
            elif str(linea[0]) == TIPOS_REG[3]:
                cbtasoc = leer(linea, CMP_ASOC)
                if 'cbte_punto_vta' in cbtasoc:
                    cbtasoc['tipo'] = cbtasoc['cbte_tipo']
                    cbtasoc['pto_vta'] = cbtasoc['cbte_punto_vta']
                    cbtasoc['nro'] = cbtasoc['cbte_nro']
                cbtasocs.append(cbtasoc)
            elif str(linea[0]) == TIPOS_REG[4]:
                detalle = leer(linea, DETALLE)
                detalles.append(detalle)
                if 'imp_subtotal' not in detalle:
                    detalle['imp_subtotal'] = detalle['importe']
            else:
                print("Tipo de registro incorrecto:", linea[0])

    if informar_caea:
        if '/testing' in sys.argv:
            encabezado['cae'] = '21353598240916'
            encabezado['fch_venc_cae'] = '2011-09-15'
        encabezado['caea'] = encabezado['cae']

    if 'imp_subtotal' not in encabezado:
        encabezado['imp_subtotal'] = encabezado['imp_neto'] + \
                encabezado['imp_tot_conc']

    ws.CrearFactura(**encabezado)
    for detalle in detalles:
        ws.AgregarItem(**detalle)
    for tributo in tributos:
        ws.AgregarTributo(**tributo)
    for iva in ivas:
        ws.AgregarIva(**iva)
    for cbtasoc in cbtasocs:
        ws.AgregarCmpAsoc(**cbtasoc)

    if DEBUG:
        print(
            '\n'.join(["%s='%s'" % (k, str(v)) for k, v in ws.factura.items()])
        )
    if not DEBUG or raw_input("Facturar?") == "S":
        if not informar_caea:
            cae = ws.AutorizarComprobante()
            dic = ws.factura
        else:
            cae = ws.InformarComprobanteCAEA()
            dic = ws.factura
        dic.update({
            'cae': cae,
            'fch_venc_cae': ws.Vencimiento,
            'resultado': ws.Resultado,
            'motivos_obs': ws.Obs,
            'err_code': ws.ErrCode,
            'err_msg': ws.ErrMsg,
            'reproceso': ws.Reproceso,
            'emision_tipo': ws.EmisionTipo,
            })
        escribir_factura(dic, salida)
        print(
            "NRO:", dic['cbt_desde'], "Resultado:", dic['resultado'], "%s:" % (
                ws.EmisionTipo, dic['cae'], "Obs:",
                dic['motivos_obs'].encode("ascii", "ignore"), "Err:",
                dic['err_msg'].encode("ascii", "ignore"), "Reproceso:",
                dic['reproceso']
            )
        )


def escribir_factura(dic, archivo, agrega=False):
    dic['tipo_reg'] = TIPOS_REG[0]
    dic['cbte_nro'] = dic.get('cbt_desde')
    archivo.write(escribir(dic, ENCABEZADO, contraer_fechas=True))
    if 'tributos' in dic:
        for it in dic['tributos']:
            it['tipo_reg'] = TIPOS_REG[1]
            archivo.write(escribir(it, TRIBUTO))
    if 'iva' in dic:
        for it in dic['iva']:
            it['tipo_reg'] = TIPOS_REG[2]
            archivo.write(escribir(it, IVA))
    if 'cbtes_asoc' in dic:
        for it in dic['cbtes_asoc']:
            it['tipo_reg'] = TIPOS_REG[3]
            archivo.write(escribir(it, CMP_ASOC))
    if 'detalles' in dic:
        for it in dic['detalles']:
            it['tipo_reg'] = TIPOS_REG[4]
            it['importe'] = it['imp_subtotal']
            archivo.write(escribir(it, DETALLE))

    if '/dbf' in sys.argv:
        formatos = [
            ('Encabezado', ENCABEZADO, [dic]),
            ('Tributo', TRIBUTO, dic.get('tributos', [])),
            ('Iva', IVA, dic.get('iva', [])),
            ('Comprobante Asociado', CMP_ASOC, dic.get('cbtes_asoc', [])),
            ('Detalles', DETALLE, dic.get('detalles', []))
        ]
        guardar_dbf(formatos, agrega, conf_dbf)


def depurar_xml(client):
    global wsmtxca_xml_dir
    fecha = time.strftime("%Y%m%d%H%M%S")
    f = open(os.path.join(wsmtxca_xml_dir, "request-%s.xml" % fecha), "w")
    f.write(client.xml_request)
    f.close()
    f = open(os.path.join(wsmtxca_xml_dir, "response-%s.xml" % fecha), "w")
    f.write(client.xml_response)
    f.close()


if __name__ == "__main__":
    if '/ayuda' in sys.argv:
        print(LICENCIA)
        print
        print("Opciones: ")
        print(" /ayuda: este mensaje")
        print(" /dummy: consulta estado de servidores")
        print(
            " /prueba: genera y autoriza una factura de prueba \
            (no usar en producci�n!)"
        )
        print(" /ult: consulta �ltimo n�mero de comprobante")
        print(" /debug: modo depuraci�n (detalla y confirma las operaciones)")
        print(
            " /formato: muestra el formato de los archivos de entrada/salida")
        print(
            " /get: recupera datos de un comprobante autorizado previamente \
            (verificaci�n)")
        print(
            " /xml: almacena los requerimientos y respuestas XML (depuraci�n)")
        print(" /dbf: lee y almacena la informaci�n en tablas DBF")
        print
        print(
            "Ver rece.ini para par�metros de configuraci�n \
            (URL, certificados, etc.)")
        sys.exit(0)

    if '/debug'in sys.argv:
        DEBUG = True
        print("VERSION", __version__, "HOMO", HOMO)

    # si se pasa el archivo de configuraci�n por par�metro, confirmar que
    # exista
    # y descartar que sea una opci�n
    if len(sys.argv) > 1 and \
       (sys.argv[1][0] not in "-/" or os.path.exists(sys.argv[1])):
        CONFIG_FILE = sys.argv.pop(1)
    if DEBUG:
        print("CONFIG_FILE:", CONFIG_FILE)

    config = SafeConfigParser()
    config.read(CONFIG_FILE)
    cert = config.get('WSAA', 'CERT')
    privatekey = config.get('WSAA', 'PRIVATEKEY')
    cuit = config.get('WSMTXCA', 'CUIT')
    if '/entrada' in sys.argv:
        entrada = sys.argv[sys.argv.index("/entrada")+1]
    else:
        entrada = config.get('WSMTXCA', 'ENTRADA')
    salida = config.get('WSMTXCA', 'SALIDA')

    if config.has_option('WSAA', 'URL') and not HOMO:
        wsaa_url = config.get('WSAA', 'URL')
    else:
        wsaa_url = None
    if config.has_option('WSMTXCA', 'URL') and not HOMO:
        wsmtxca_url = config.get('WSMTXCA', 'URL')
    else:
        wsmtxca_url = None

    if config.has_option('WSMTXCA', 'REPROCESAR'):
        wsmtxca_reprocesar = config.get('WSMTXCA', 'REPROCESAR') == 'S'
    else:
        wsmtxca_reprocesar = None

    if config.has_option('WSMTXCA', 'XML_DIR'):
        wsmtxca_xml_dir = config.get('WSMTXCA', 'XML_DIR')
    else:
        wsmtxca_xml_dir = "."

    if config.has_section('DBF'):
        conf_dbf = dict(config.items('DBF'))
        if DEBUG:
            print("conf_dbf", conf_dbf)
    else:
        conf_dbf = {}

    if '/xml'in sys.argv:
        XML = True

    if DEBUG:
        print(
            "wsaa_url %s\nwsmtxca_url %s\ncuit %s" % (
                wsaa_url, wsmtxca_url, cuit)
        )

    try:
        ws = wsmtx.WSMTXCA()
        ws.Conectar("", wsmtxca_url)
        ws.Cuit = cuit
        if wsmtxca_reprocesar is not None:
            ws.Reprocesar = wsmtxca_reprocesar

        if '/dummy' in sys.argv:
            print("Consultando estado de servidores...")
            ws.Dummy()
            print("AppServerStatus", ws.AppServerStatus)
            print("DbServerStatus", ws.DbServerStatus)
            print("AuthServerStatus", ws.AuthServerStatus)
            sys.exit(0)

        if '/formato' in sys.argv:
            print("Formato:")
            for msg, formato in [
                ('Encabezado', ENCABEZADO),
                ('Tributo', TRIBUTO),
                ('Iva', IVA),
                ('Comprobante Asociado', CMP_ASOC),
                ('Detalle', DETALLE)
            ]:
                comienzo = 1
                print("== %s ==" % msg)
                for fmt in formato:
                    clave, longitud, tipo = fmt[0:3]
                    if isinstance(longitud, tuple):
                        longitud, dec = longitud
                    else:
                        dec = len(fmt) > 3 and fmt[3] or 2
                    print(
                        " * Campo: %-20s Posici�n: %3d Longitud: %4d Tipo: \
                        %s Decimales: %s" % (
                            clave, comienzo, longitud, tipo, dec))
                    comienzo += longitud
            sys.exit(0)

        # obteniendo el TA
        from wsaa import WSAA
        wsaa = WSAA()
        ta = wsaa.Autenticar("wsmtxca", cert, privatekey, wsaa_url)
        if not ta:
            sys.exit("Imposible autenticar con WSAA: %s" % wsaa.Excepcion)
        ws.SetTicketAcceso(ta)

        if '/puntosventa' in sys.argv:
            print("Consultando puntos de venta CAE...")
            print('\n'.join(ws.ConsultarPuntosVentaCAE()))
            print("Consultando puntos de venta CAEA...")
            if "--testing" in sys.argv:
                ws.LoadTestXML("tests/wsmtx_ptosvta_caea_resp.xml")
            print('\n'.join(ws.ConsultarPuntosVentaCAEA()))
            sys.exit(0)

        if '/prueba' in sys.argv:
            # generar el archivo de prueba para la pr�xima factura
            tipo_cbte = 6
            punto_vta = 4000
            cbte_nro = ws.ConsultarUltimoComprobanteAutorizado(
                tipo_cbte, punto_vta)
            fecha = datetime.datetime.now().strftime("%Y-%m-%d")
            concepto = 3
            tipo_doc = 80
            nro_doc = "30000000007"
            cbte_nro = long(cbte_nro) + 1
            cbt_desde = cbte_nro
            cbt_hasta = cbt_desde
            imp_total = "121.00"
            imp_tot_conc = "0.00"
            imp_neto = "100.00"
            imp_trib = "0.00"
            imp_op_ex = "0.00"
            imp_subtotal = "100.00"
            fecha_cbte = fecha
            fecha_venc_pago = fecha
            # Fechas del per�odo del servicio facturado (solo si concepto = 1?)
            fecha_serv_desde = fecha
            fecha_serv_hasta = fecha
            moneda_id = 'PES'
            moneda_ctz = '1.000'
            obs = "Observaciones Comerciales, libre"

            ws.CrearFactura(concepto, tipo_doc, nro_doc, tipo_cbte, punto_vta,
                            cbt_desde, cbt_hasta, imp_total, imp_tot_conc,
                            imp_neto, imp_subtotal, imp_trib, imp_op_ex,
                            fecha_cbte, fecha_venc_pago, fecha_serv_desde,
                            fecha_serv_hasta, moneda_id, moneda_ctz, obs)

            if tipo_cbte not in (1, 2, 6, 7):
                tipo = 1
                pto_vta = 2
                nro = 1234
                ws.AgregarCmpAsoc(tipo, pto_vta, nro)

            tributo_id = 99
            desc = 'Impuesto Municipal Matanza'
            base_imp = 100
            alic = 1
            importe = 1
            # ws.AgregarTributo(tributo_id, desc, base_imp, alic, importe)

            iva_id = 5  # 21%
            base_imp = 100
            importe = 21
            ws.AgregarIva(iva_id, base_imp, importe)

            u_mtx = 123456
            cod_mtx = 1234567890123
            codigo = "P0001"
            ds = "Descripcion del producto P0001"
            qty = 1.00
            umed = 7
            if tipo_cbte in (6, 7, 8):
                precio = 121.00
            else:
                precio = 100.00
            bonif = 0.00
            iva_id = 5
            imp_iva = 21.00
            imp_subtotal = 121.00
            ws.AgregarItem(
                u_mtx, cod_mtx, codigo, ds, qty, umed, precio, bonif,
                iva_id, imp_iva, imp_subtotal)

            if tipo_cbte not in (6, 7, 8):
                ws.AgregarItem(u_mtx, cod_mtx, codigo, "PRUEBA", 1, 7, 1.00, 0,
                               iva_id, 0.21, 1.21)
                ws.AgregarItem(1, "DESC", "DESC", "Descuento", 0, 99, 0, 0,
                               iva_id, 0.21, 1.21)
            else:
                ws.AgregarItem(u_mtx, cod_mtx, codigo, "PRUEBA", 1, 7, 1.21, 0,
                               iva_id, 0.0, 1.21)
                ws.AgregarItem(1, "DESC", "DESC", "Descuento", 0, 99, 0, 0,
                               iva_id, 0.0, 1.21)

            f_entrada = open(entrada, "w")

            if DEBUG:
                print(ws.factura)

            dic = ws.factura
            escribir_factura(dic, f_entrada, agrega=True)
            f_entrada.close()

        if '/ult' in sys.argv:
            print("Consultar ultimo numero:")
            i = sys.argv.index("/ult")
            if i+2 < len(sys.argv):
                tipo_cbte = int(sys.argv[i+1])
                punto_vta = int(sys.argv[i+2])
            else:
                tipo_cbte = int(raw_input("Tipo de comprobante: "))
                punto_vta = int(raw_input("Punto de venta: "))
            ult_cbte = ws.ConsultarUltimoComprobanteAutorizado(
                tipo_cbte, punto_vta)
            print("Ultimo numero: ", ult_cbte)
            depurar_xml(ws.client)
            escribir_factura({'tipo_cbte': tipo_cbte,
                              'punto_vta': punto_vta,
                              'cbt_desde': ult_cbte,
                              'fecha_cbte': ws.FechaCbte,
                              }, open(salida, "w"))
            sys.exit(0)

        if '/get' in sys.argv:
            print("Recuperar comprobante:")
            i = sys.argv.index("/get")
            if i+3 < len(sys.argv):
                tipo_cbte = int(sys.argv[i+1])
                punto_vta = int(sys.argv[i+2])
                cbte_nro = int(sys.argv[i+3])
            else:
                tipo_cbte = int(raw_input("Tipo de comprobante: "))
                punto_vta = int(raw_input("Punto de venta: "))
                cbte_nro = int(raw_input("Numero de comprobante: "))
            ws.ConsultarComprobante(tipo_cbte, punto_vta, cbte_nro)

            print("FechaCbte = ", ws.FechaCbte)
            print("CbteNro = ", ws.CbteNro)
            print("PuntoVenta = ", ws.PuntoVenta)
            print("ImpTotal =", ws.ImpTotal)
            print("CAE = ", ws.CAE)
            print("Vencimiento = ", ws.Vencimiento)
            print("EmisionTipo = ", ws.EmisionTipo)

            depurar_xml(ws.client)
            escribir_factura({'tipo_cbte': tipo_cbte,
                              'punto_vta': ws.PuntoVenta,
                              'cbt_desde': ws.CbteNro,
                              'fecha_cbte': ws.FechaCbte,
                              'imp_total': ws.ImpTotal,
                              'cae': ws.CAE,
                              'fch_venc_cae': ws.Vencimiento,
                              'emision_tipo': ws.EmisionTipo,
                              }, open(salida, "w"))

            sys.exit(0)

        if '/solicitarcaea' in sys.argv:
            if len(sys.argv) > sys.argv.index("/solicitarcaea")+1:
                periodo = sys.argv[sys.argv.index("/solicitarcaea")+1]
                orden = sys.argv[sys.argv.index("/solicitarcaea")+2]
            else:
                periodo = raw_input("Periodo (a�o-mes, ej 201108): ")
                orden = raw_input("Orden (quincena, 1 u 2): ")

            if DEBUG:
                print(
                    "Solicitando CAEA para periodo %s orden %s" % (
                        periodo, orden)
                )

            caea = ws.SolicitarCAEA(periodo, orden)
            print("CAEA:", caea)

            if ws.Errores:
                print("Errores:")
                for error in ws.Errores:
                    print(error)

            depurar_xml(ws.client)

            if not caea:
                if DEBUG:
                    print(
                        "Consultando CAEA para periodo %s orden %s" % (
                            periodo, orden)
                    )
                caea = ws.ConsultarCAEA(periodo, orden)
                print("CAEA:", caea)

            if DEBUG:
                print("Periodo:", ws.Periodo)
                print("Orden:", ws.Orden)
                print("FchVigDesde:", ws.FchVigDesde)
                print("FchVigHasta:", ws.FchVigHasta)
                print("FchTopeInf:", ws.FchTopeInf)
                print("FchProceso:", ws.FchProceso)

            escribir_factura(
                {'cae': caea,
                 'emision_tipo': "CAEA"},
                open(salida, "w")
            )

            sys.exit(0)

        if '/consultarcaea' in sys.argv:
            periodo = raw_input("Periodo: ")
            orden = raw_input("Orden: ")

            if DEBUG:
                print(
                    "Consultando CAEA para periodo %s orden %s" % (
                        periodo, orden)
                )

            caea = ws.ConsultarCAEA(periodo, orden)
            print("CAEA:", caea)

            if ws.Errores:
                print("Errores:")
                for error in ws.Errores:
                    print(error)

            if DEBUG:
                print("Periodo:", ws.Periodo)
                print("Orden:", ws.Orden)
                print("FchVigDesde:", ws.FchVigDesde)
                print("FchVigHasta:", ws.FchVigHasta)
                print("FchTopeInf:", ws.FchTopeInf)
                print("FchProceso:", ws.FchProceso)
            sys.exit(0)

        if '/informarcaeanoutilizado' in sys.argv:
            caea = raw_input("CAEA: ")
            if DEBUG:
                print("Informando CAEA no utilizado: %s" % (caea, ))
            ok = ws.InformarCAEANoUtilizado(caea)
            print("Resultado:", ok)
            if ws.Errores:
                print("Errores:")
                for error in ws.Errores:
                    print(error)
            sys.exit(0)

        if '/informarcaeanoutilizadoptovta' in sys.argv:
            caea = raw_input("CAEA: ")
            pto_vta = raw_input("Punto de Venta: ")
            if DEBUG:
                print(
                    "Informando CAEA no utilizado: %s pto_vta %s" % (
                        caea, pto_vta)
                )
            ok = ws.InformarCAEANoUtilizadoPtoVta(caea, pto_vta)
            print("Resultado:", ok)
            if ws.Errores:
                print("Errores:")
                for error in ws.Errores:
                    print(error)
            sys.exit(0)

        if '/consultarptosvtacaeanoinformados' in sys.argv:
            caea = raw_input("CAEA: ")
            if DEBUG:
                print("Consultando PtosVta CAEA: %s" % (caea))
            ptos_vta = ws.ConsultarPtosVtaCAEANoInformados(caea)
            print("Resultado:", '\n'.join(ptos_vta))
            if ws.Errores:
                print("Errores:")
                for error in ws.Errores:
                    print(error)
            sys.exit(0)

        f_entrada = f_salida = None
        try:
            f_entrada = open(entrada, "r")
            f_salida = open(salida, "w")
            try:
                if DEBUG:
                    print("Autorizando usando entrada:", entrada)
                autorizar(ws, f_entrada, f_salida, '/informarcaea' in sys.argv)
            except SoapFault:
                XML = True
                raise
        finally:
            if f_entrada is not None:
                f_entrada.close()
            if f_salida is not None:
                f_salida.close()
            if XML:
                depurar_xml(ws.client)
        sys.exit(0)

    except SoapFault as e:
        print(e.faultcode, e.faultstring.encode("ascii", "ignore"))
        sys.exit(3)
    except Exception as e:
        e_str = unicode(e).encode("ascii", "ignore")
        if not e_str:
            e_str = repr(e)
        print(e_str)
        escribir_factura(
            {'err_msg': e_str},
            open(salida, "w")
        )
        ex = traceback.format_exception(
            sys.exc_type, sys.exc_value, sys.exc_traceback)
        open("traceback.txt", "wb").write('\n'.join(ex))

        if DEBUG:
            raise
        sys.exit(5)



#!/usr/bin/python
# -*- coding: latin-1 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

"Módulo para consultar percepciones / retenciones ARBA IIBB"

__author__ = "Mariano Reingart (reingart@gmail.com)"
__copyright__ = "Copyright (C) 2010 Mariano Reingart"
__license__ = "LGPL 3.0"
__version__ = "1.01b"

import md5
import os
import sys
import tempfile
import traceback
from py3simplesoap.simplexml import SimpleXMLElement
import logging

from .utils import WebClient


log = logging.getLogger()
HOMO = False
CACERT = "conf/arba.crt"   # establecimiento de canal seguro (en producción)

URL = "https://dfe.test.arba.gov.ar/DomicilioElectronico/SeguridadCliente/"
"dfeServicioConsulta.do"  # testing
# URL = "https://dfe.arba.gov.ar/DomicilioElectronico/SeguridadCliente/"
# "dfeServicioConsulta.do"  # produccion


XML_ENTRADA_BASE = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<CONSULTA-ALICUOTA>
    <cantidadContribuyentes>1</cantidadContribuyentes>
      <contribuyentes class="list">
        <contribuyente>
        </contribuyente>
      </contribuyentes>
</CONSULTA-ALICUOTA>
"""


class IIBB:
    "Interfaz para el servicio de IIBB ARBA"
    _public_methods_ = ['Conectar', 'ConsultarContribuyentes',
                        'LeerContribuyente', 'LeerErrorValidacion',
                        'AnalizarXml', 'ObtenerTagXml']
    _public_attrs_ = ['Usuario', 'Password', 'XmlResponse',
                      'Version', 'Excepcion', 'Traceback', 'InstallDir',
                      'NumeroComprobante', 'CantidadContribuyentes',
                      'CodigoHash', 'CuitContribuyente', 'AlicuotaPercepcion',
                      'AlicuotaRetencion', 'GrupoPercepcion', 'GrupoRetencion',
                      'TipoError', 'CodigoError', 'MensajeError']

    _reg_progid_ = "IIBB"
    _reg_clsid_ = "{2C7E29D2-0C99-49D8-B04B-A16B807BB123}"

    Version = "%s %s" % (__version__, HOMO and 'Homologación' or '')

    def __init__(self):
        self.Usuario = self.Password = None
        self.TipoError = self.CodigoError = self.MensajeError = ""
        self.InstallDir = INSTALL_DIR
        self.client = None
        self.xml = None
        self.limpiar()

    def limpiar(self):
        self.NumeroComprobante = self.CodigoHash = ""
        self.CantidadContribuyentes = 0
        self.CuitContribuyente = ""
        self.AlicuotaPercepcion = 0
        self.AlicuotaRetencion = 0
        self.GrupoPercepcion = 0
        self.GrupoRetencion = 0
        self.contribuyentes = []
        self.errores = []
        self.XmlResponse = ""
        self.Excepcion = self.Traceback = ""
        self.TipoError = self.CodigoError = self.MensajeError = ""

    def Conectar(
        self, url=None, proxy="", wrapper=None, cacert=None, trace=False,
        testing=""
    ):
        if HOMO or not url:
            url = URL
        self.client = WebClient(location=url, trace=trace, cacert=cacert)
        self.testing = testing

    def ConsultarContribuyentes(
        self, fecha_desde, fecha_hasta, cuit_contribuyente
    ):
        "Realiza la consulta remota a ARBA, estableciendo los resultados"
        self.limpiar()
        try:

            self.xml = SimpleXMLElement(XML_ENTRADA_BASE)
            self.xml.fechaDesde = fecha_desde
            self.xml.fechaHasta = fecha_hasta
            self.xml.contribuyentes.contribuyente.cuitContribuyente = cuit_contribuyente

            xml = self.xml.as_xml()
            self.CodigoHash = md5.md5(xml).hexdigest()
            nombre = "DFEServicioConsulta_%s.xml" % self.CodigoHash

            # guardo el xml en el archivo a enviar y luego lo re-abro:
            archivo = open(os.path.join(tempfile.gettempdir(), nombre), "w")
            archivo.write(xml)
            archivo.close()
            archivo = open(os.path.join(tempfile.gettempdir(), nombre), "r")

            if not self.testing:
                response = self.client(user=self.Usuario,
                                       password=self.Password,
                                       file=archivo)
            else:
                response = open(self.testing).read()
            self.XmlResponse = response
            self.xml = SimpleXMLElement(response)
            if 'tipoError' in self.xml:
                self.TipoError = str(self.xml.tipoError)
                self.CodigoError = str(self.xml.codigoError)
                self.MensajeError = str(self.xml.mensajeError)\
                    .decode('latin1')\
                    .encode("ascii", "replace")
            if 'numeroComprobante' in self.xml:
                self.NumeroComprobante = str(self.xml.numeroComprobante)
                self.CantidadContribuyentes = int(
                    self.xml.cantidadContribuyentes
                )
                if 'contribuyentes' in self.xml:
                    for contrib in self.xml.contribuyente:
                        c = {
                            'CuitContribuytente': str(
                                contrib.cuitContribuyente
                            ),
                            'AlicuotaPercepcion': str(
                                contrib.alicuotaPercepcion
                            ),
                            'AlicuotaRetencion': str(
                                contrib.alicuotaRetencion
                            ),
                            'GrupoPercepcion': str(contrib.grupoPercepcion),
                            'GrupoRetencion': str(contrib.grupoRetencion),
                            'Errores': [],
                            }
                        self.contribuyentes.append(c)
                    # establecer valores del primer contrib (sin eliminarlo)
                    self.LeerContribuyente(pop=False)
            return True
        except Exception as e:
            log.warn(e)
            ex = traceback.format_exception(
                sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2]
            )
            self.Traceback = ''.join(ex)
            try:
                self.Excepcion = traceback.format_exception_only(
                    sys.exc_info()[0], sys.exc_info()[1]
                )[0]
            except:
                self.Excepcion = "<no disponible>"
            return False

    def LeerContribuyente(self, pop=True):
        "Leeo el próximo contribuyente"
        # por compatibilidad hacia atras, la primera vez no remueve de la lista
        # (llamado de ConsultarContribuyentes con pop=False)
        if self.contribuyentes:
            contrib = self.contribuyentes[0]
            if pop:
                del self.contribuyentes[0]
        else:
            contrib = {}
        self.CuitContribuyente = contrib.get('CuitContribuytente', "")
        self.AlicuotaPercepcion = contrib.get('AlicuotaPercepcion', "")
        self.AlicuotaRetencion = contrib.get('AlicuotaRetencion', "")
        self.GrupoPercepcion = contrib.get('GrupoPercepcion', "")
        self.GrupoRetencion = contrib.get('GrupoRetencion', "")
        self.errores = contrib.get('Errores', [])
        return len(contrib) > 0

    def LeerErrorValidacion(self):
        if self.errores:
            error = self.errores.pop()
            self.TipoError = ""
            self.CodigoError = error[0]
            self.MensajeError = error[1]
            return True
        else:
            self.TipoError = ""
            self.CodigoError = ""
            self.MensajeError = ""
            return False

    def AnalizarXml(self, xml=""):
        "Analiza un mensaje XML (por defecto la respuesta)"
        try:
            if not xml:
                xml = self.XmlResponse
            self.xml = SimpleXMLElement(xml)
            return True
        except Exception as e:
            self.Excepcion = "%s" % (e)
            return False

    def ObtenerTagXml(self, *tags):
        "Busca en el Xml analizado y devuelve el tag solicitado"
        # convierto el xml a un objeto
        try:
            if self.xml:
                xml = self.xml
                # por cada tag, lo busco segun su nombre o posición
                for tag in tags:
                    xml = xml(tag)  # atajo a getitem y getattr
                # vuelvo a convertir a string el objeto xml encontrado
                return str(xml)
        except Exception as e:
            self.Excepcion = "%s" % (e)


# busco el directorio de instalación (global para que no cambie si usan
# otra dll)
if not hasattr(sys, "frozen"):
    basepath = __file__
elif sys.frozen == 'dll':
    import win32api
    basepath = win32api.GetModuleFileName(sys.frozendllhandle)
else:
    basepath = sys.executable
INSTALL_DIR = os.path.dirname(os.path.abspath(basepath))


if __name__ == "__main__":

    if "--register" in sys.argv or "--unregister" in sys.argv:
        import win32com.server.register
        win32com.server.register.UseCommandLine(IIBB)
        sys.exit(0)
    elif len(sys.argv) < 6:
        print("Se debe especificar usuario, clave, fecha desde/hasta y cuit\
                como argumentos!")
        sys.exit(1)

    iibb = IIBB()
    iibb.Usuario = sys.argv[1]   # 20267565393
    iibb.Password = sys.argv[2]  # 23456
    fecha_desde = sys.argv[3]   # 20150301
    fecha_hasta = sys.argv[4]   # 20150331
    cuit_contribuyente = sys.argv[5]          # 30123456780

    if '--testing' in sys.argv:
        test_response = "iibb_response.xml"
        # test_response = "iibb_response_2_errores.xml"
    else:
        test_response = ""

    if not HOMO:
        for i, arg in enumerate(sys.argv):
            if arg.startswith("--prod"):
                URL = URL.replace("https://dfe.test.arba.gov.ar/",
                                  "https://dfe.arba.gov.ar/")
                print("Usando URL:", URL)
                break
            if arg.startswith("https"):
                URL = arg
                print("Usando URL:", URL)
                break

    iibb.Conectar(
        URL, trace='--trace' in sys.argv, cacert=None, testing=test_response
    )
    iibb.ConsultarContribuyentes(fecha_desde, fecha_hasta, cuit_contribuyente)

    if iibb.Excepcion:
        print("Excepcion:", iibb.Excepcion)
        print("Traceback:", iibb.Traceback)

    # datos generales:
    print("Numero Comprobante:", iibb.NumeroComprobante)
    print("Codigo HASH:", iibb.CodigoHash)
    print(
        "Error General:", iibb.TipoError, "|", iibb.CodigoError,
        "|", iibb.MensajeError
    )

    # recorro los contribuyentes devueltos e imprimo sus datos por cada uno:
    while iibb.LeerContribuyente():
        print("CUIT Contribuytente:", iibb.CuitContribuyente)
        print("AlicuotaPercepcion:", iibb.AlicuotaPercepcion)
        print("AlicuotaRetencion:", iibb.AlicuotaRetencion)
        print("GrupoPercepcion:", iibb.GrupoPercepcion)
        print("GrupoRetencion:", iibb.GrupoRetencion)

    # Ejemplos de uso ObtenerTagXml
    if False:
        print("desde", iibb.ObtenerTagXml('fechaDesde'))
        print("hasta", iibb.ObtenerTagXml('fechaHasta'))
        print(
            "cuit", iibb.ObtenerTagXml(
                'contribuyentes', 'contribuyente', 0, 'cuitContribuyente'
            )
        )
        print(
            "alicuota", iibb.ObtenerTagXml(
                'contribuyentes', 'contribuyente', 0, 'alicuotapercepcion'
            )
        )
